// Copyright 1998, Jason Hunter <jason_hunter@acm.org>
// Permission is granted to use this file for testing only.
// Contact the author before employing it for any other use.
// The final version will be made freely available.
// (I just don't want this prototype making the rounds, ya know!)

import java.io.*;
import java.util.*;
import javax.servlet.*;

public class MultipartRequest {

  private static final int DEFAULT_MAX_POST_SIZE = 1024 * 1024;  // 1 Meg

  private ServletRequest req;
  private File dir;
  private int maxSize;

  private Hashtable parameters = new Hashtable();  // name - value
  private Hashtable files = new Hashtable();       // name - UploadedFile

  public MultipartRequest(ServletRequest request,
                          String saveDirectory) throws IOException {
    this(request, saveDirectory, DEFAULT_MAX_POST_SIZE);
  }

  public MultipartRequest(ServletRequest request,
                          String saveDirectory,
                          int maxPostSize) throws IOException {
    // Sanity check values
    if (request == null)
      throw new IllegalArgumentException("request cannot be null");
    if (saveDirectory == null)
      throw new IllegalArgumentException("saveDirectory cannot be null");
    if (maxPostSize <= 0) {
      throw new IllegalArgumentException("maxPostSize must be positive");
    }

    // Save the request, dir, and max size
    req = request;
    dir = new File(saveDirectory);
    maxSize = maxPostSize;

    // Check saveDirectory is truly a directory
    if (!dir.isDirectory())
      throw new IllegalArgumentException("Not a directory: " + saveDirectory);

    // Check saveDirectory is writable
    if (!dir.canWrite())
      throw new IllegalArgumentException("Not writable: " + saveDirectory);

    // Now parse the request saving data to parameters and files;
    // write the file contents to the saveDirectory
    parseRequest(req, dir, maxSize);
  }

  public Enumeration getParameterNames() {
    return parameters.keys();
  }

  public Enumeration getFileNames() {
    return files.keys();
  }

  public String getParameter(String name) {
    try {
      String param = (String)parameters.get(name);
      if (param.equals("")) return null;
      return param;
    }
    catch (Exception e) {
      return null;
    }
  }

  public String getFilesystemName(String name) {
    try {
      UploadedFile file = (UploadedFile)files.get(name);
      return file.getFilesystemName();  // may be null
    }
    catch (Exception e) {
      return null;
    }
  }

  public String getContentType(String name) {
    try {
      UploadedFile file = (UploadedFile)files.get(name);
      return file.getContentType();  // may be null
    }
    catch (Exception e) {
      return null;
    }
  }

  public File getFile(String name) {
    try {
      UploadedFile file = (UploadedFile)files.get(name);
      return file.getFile();  // may be null
    }
    catch (Exception e) {
      return null;
    }
  }

  // This method is a monster.  It should really be broken into easier 
  // to manage chunks, but that's a project for another day.
  protected void parseRequest(ServletRequest req, File dir, int maxSize) 
                                     throws IOException {

    // Check the content type to make sure it's "multipart/form-data"
    String type = req.getContentType();
    if (type == null || 
        !type.toLowerCase().startsWith("multipart/form-data")) {
      throw new IOException("Posted content type isn't multipart/form-data");
    }

    // Check the content length to prevent denial of service attacks
    int length = req.getContentLength();
    if (length > maxSize) {
      throw new IOException("Posted content length of " + length + 
                            " exceeds limit of " + maxSize);
    }

    // Get the boundary string; it's included in the content type.
    // Should look something like "------------------------12012133613061"
    String boundary = getBoundary(type);
    if (boundary == null) {
      throw new IOException("Separation boundary was not specified");
    }
System.out.println("boundary: " + boundary);

    // Get the input stream we'll read from and a byte array to read into
    ServletInputStream in = req.getInputStream();
    byte[] buf = new byte[1024 * 8];  // 8K

    // Read the first line, should be the first boundary
    int result = in.readLine(buf, 0, buf.length);
    if (result == -1) {
      throw new IOException("Corrupt form data: premature ending");
    }

    // Convert the line to a string
    String line = new String(buf, 0, result, "ISO-8859-1");

    // Verify that the line is the boundary
    if (!line.startsWith(boundary)) {
      throw new IOException("Corrupt form data: no leading boundary");
    }

    // Now that we're just beyond the first boundary, loop over each part
    while (true) {
      // Read the first line, should look like this:
      // content-disposition: form-data; name="field1"; filename="file1.txt"
      result = in.readLine(buf, 0, buf.length);
      if (result == -1) {
        // No parts left, we're done
        return;
      }

      // Convert the line to a lowercase string without the ending "\r\n"
      // Keep the original line for error messages and for variable names.
      String origline = new String(buf, 0, result - 2, "ISO-8859-1");
      line = origline.toLowerCase();
System.out.println("line: " + line);

      // Get the content disposition, should be "form-data"
      int start = line.indexOf("content-disposition: ");
      int end = line.indexOf(";");
      if (start == -1 || end == -1) {
        throw new IOException("Content disposition corrupt: " + origline);
      }
      String disposition = line.substring(start + 21, end);
      if (!disposition.equals("form-data")) {
        throw new IOException("Invalid content disposition: " + disposition);
      }
System.out.println("disposition: " + disposition);

      // Get the field name
      start = line.indexOf("name=\"", end);  // start at last semicolon
      end = line.indexOf("\"", start + 7);   // skip name=\"
      if (start == -1 || end == -1) {
        throw new IOException("Content disposition corrupt: " + origline);
      }
      String name = origline.substring(start + 6, end);
System.out.println("field name: " + name);

      // Get the filename, if given
      String filename = null;
      start = line.indexOf("filename=\"", end + 2);  // start after name
      end = line.indexOf("\"", start + 10);          // skip filename=\"
      if (start != -1 && end != -1) {                // note the !=
        filename = origline.substring(start + 10, end);
System.out.println("filename (maybe with path): " + filename);
        // The filename may contain a full path.  Cut to just the filename.
        int slash =
          Math.max(filename.lastIndexOf('/'), filename.lastIndexOf('\\'));
        if (slash > -1) {
          filename = filename.substring(slash + 1);  // past last slash
        }
        if (filename.equals("")) filename = "unknown"; // sanity check
      }
System.out.println("filename: " + filename);

      // Now onto the next line.  This will either be empty 
      // or contain a Content-Type and then an empty line.
      result = in.readLine(buf, 0, buf.length);
      if (result == -1) {
        // No parts left, we're done
        return;
      }

      // Convert the line to a lowercase string, cut off the "\r\n"
      line = new String(buf, 0, result - 2, "ISO-8859-1").toLowerCase();

      // Get the content type, if any
      String contentType = null;
      if (line.startsWith("content-type")) {
        start = line.indexOf(" ");
        if (start == -1) {
          throw new IOException("Content type corrupt: " + origline);
        }
        contentType = line.substring(start + 1);
        // Eat the empty line
        result = in.readLine(buf, 0, buf.length);
        if (result == -1) {
          // No parts left, we're done
          return;
        }
        else if (result != 2) {  // 2 for "\r\n"
          throw new 
            IOException("Malformed line after content type: " + origline);
        }
      }
      else if (line.length() != 0) {  // no content type, so should be empty
        throw new IOException("Malformed line after disposition: " + origline);
      }
      else {
        // Assume a default content type
        contentType = "application/octet-stream";
      }
System.out.println("Content type: " + contentType);

      // Now, finally, we read the content (end after reading the boundary)
      if (filename == null) {
        // This is a parameter
        String value = readParameter(in, boundary);
        parameters.put(name, value);
      }
      else {
        // This is a file
        readAndSaveFile(in, boundary, dir, filename);
        if (filename.equals("unknown")) {
          files.put(name, new UploadedFile(null, null, null));
        }
        else {
          files.put(name,
            new UploadedFile(dir.toString(), filename, contentType));
        }
      }
    }
  }

  private String getBoundary(String line) {
    int index = line.indexOf("boundary=");
    if (index == -1) {
      return null;
    }
    String boundary = line.substring(index + 9);  // 9 for "boundary="

    // The real boundary is always preceeded by an extra "--"
    boundary = "--" + boundary;

    return boundary;
  }

  protected String readParameter(ServletInputStream in, String boundary) 
                                        throws IOException {
    StringBuffer sbuf = new StringBuffer();
    byte[] bbuf = new byte[1024 * 8];  // 8K
    int result;
    String line;

    while ((result = in.readLine(bbuf, 0, bbuf.length)) != -1) {
      line = new String(bbuf, 0, result, "ISO-8859-1");
      if (line.startsWith(boundary)) break;
      sbuf.append(line);
    }

    sbuf.setLength(sbuf.length() - 2);  // cut off the trailing "\r\n"
    return sbuf.toString();  // no URL decoding needed
  }

  protected void readAndSaveFile(ServletInputStream in,
                            String boundary,
                            File dir,
                            String filename) 
                         throws IOException {
    File f = new File(dir + File.separator + filename);
    FileOutputStream fos = new FileOutputStream(f);
    BufferedOutputStream out = new BufferedOutputStream(fos, 1024 * 8); // 8K

    byte[] bbuf = new byte[1024 * 8];  // 8K
    int result;
    String line;

    // ServletInputStream.readLine() has the annoying habit of 
    // adding a \r\n to the end of the last line.  
    // Since we want a byte-for-byte transfer, we have to cut those chars.
System.out.println("writing to: " + f.toString());
System.out.println("start: " + new Date());
System.out.flush();
    boolean rnflag = false;
    while ((result = in.readLine(bbuf, 0, bbuf.length)) != -1) {
      // Check for boundary
      if (result > 2 && bbuf[0] == '-' && bbuf[1] == '-') { // quick pre-check
        line = new String(bbuf, 0, result, "ISO-8859-1");
        if (line.startsWith(boundary)) break;
      }
      // Are we supposed to write \r\n for the last iteration?
      if (rnflag) {
        out.write('\r'); out.write('\n');
        rnflag = false;
      }
      // Write the buffer, postpone any ending \r\n
      if (result >= 2 && 
          bbuf[result - 2] == '\r' && 
          bbuf[result - 1] == '\n') {
        out.write(bbuf, 0, result - 2);  // skip the last 2 chars
        rnflag = true;  // make a note to write them on the next iteration
      }
      else {
        out.write(bbuf, 0, result);
      }
    }
    out.flush();
    out.close();
    fos.close();
System.out.println("end: " + new Date());
  }
}


class UploadedFile {

  private String dir;
  private String filename;
  private String type;

  UploadedFile(String dir, String filename, String type) {
    this.dir = dir;
    this.filename = filename;
    this.type = type;
  }

  public String getContentType() {
    return type;
  }

  public String getFilesystemName() {
    return filename;
  }

  public File getFile() {
    if (dir == null || filename == null) return null;
    else return new File(dir + File.separator + filename);
  }
}

