/**
* @author  Umesh Kulkarni
* @version 1.0
*
* Development Environment         : JDeveloper 2.0
*
* Name of the Application         : BFILESample.java
*
* Creation/ Modification History  :
*
*       ukulkarn.in       11-Jan-1999    Creation
*
* Overview of the Application     :
*  This sample application illustrates accessing of BFILEs using JDBC.
*
*  BFILE datatype allows us to store large binary objects (like image files,
*  executable files) in operating system files outside the database.
*
*  This Sample uses JDBC-Thin Driver and JDBC-Streams to access an image file
*  stored as BFILE column and displays the image in the panel.
*
*  The GUI for this sample is handle by the class BFILEFrame. (BFILEFrame.java)
*
*
*/

import java.sql.*; // Package containing JDBC classes
import java.io.*;  //Package for using Streams

//Package for Oracle Extensions to JDBC
import oracle.jdbc.driver.*;
import oracle.sql.*;

public class BFILESample {

  Connection m_connection; //Database Connection Object
  BFILEFrame m_GUI; // GUI handler for this sample

  /**
  *  Constructor. Instantiates GUI.
  **/
  public BFILESample() {
    m_GUI = new BFILEFrame(this); // Instantiate GUI
    m_GUI.setVisible(true);
  }

  /**
  *  Main entry point to the application class. Instantiates the class,
  *  sets up the database connection. Also creation of table BFILE_TABLE and
  *  insertion of an image into the table is done here.
  **/
  public static void main(String[] args) {
    BFILESample  rootFrame = new BFILESample(); // Root Frame
    rootFrame.dbConnection(); // Connects To Database
    rootFrame.createTable(); // Creates Table BFILE_TABLE
    rootFrame.insertData(); // Inserts Image Into Table BFILE_TABLE
  }

  /**
  *  Dispatches the GUI events to the appropriate method, which performs
  *  the required JDBC operations. This method is invoked from the setupListeners
  *  section of BFILEFrame.java
  **/
  public void dispatchEvent(String p_eventName){
    // Dispatch Event to appropriate Methods
    if (p_eventName.equals("SELECT"))
     selectBFILE();
    else if (p_eventName.equals("EXIT"))
     exitApplication();
  }

  /**
  *  Creates a database connection object using JDBC. Please substitute the
  *  database connection parameters with appropriate values in
  *  ConnectionParams.java
  **/
  public void dbConnection(){
    try{

      m_GUI.m_selectButton.setEnabled(false);
      m_GUI.putStatus("Trying to connect to the Database");

      // Load the Oracle JDBC Driver and register it.
      DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());

      // Form the database connect string(TNSNAMES entry) as a name-value pair
      // using the connection parameters as specified in ConnectionParams.java
      String l_dbConnectString =
          "(DESCRIPTION=(ADDRESS=(HOST="+ConnectionParams.s_hostName+")"+
          "(PROTOCOL=tcp)(PORT="+ConnectionParams.s_portNumber+"))"+
          "(CONNECT_DATA=(SID="+ConnectionParams.s_databaseSID+")))";

     // The following statement creates a database connection object
     // using the DriverManager.getConnection method. The first parameter is
     // the database URL which is constructed based on the connection parameters
     // specified in ConnectionParams.java.
     // The URL syntax is as follows:
     //      "jdbc:oracle:<driver>:@<db connection string>"
     // <driver>, can be 'thin' or 'oci8'
     // <db connect string>, is a Net8 name-value, denoting the TNSNAMES entry
     m_connection = DriverManager.getConnection(
                  "jdbc:oracle:thin:@"+l_dbConnectString,
                  ConnectionParams.s_userName,
                  ConnectionParams.s_password);

     // sets the auto-commit property for the connection to be false. By default
     // the connections always auto-commit.
     m_connection.setAutoCommit(false);

     m_GUI.putStatus("Connected to "+ConnectionParams.s_databaseSID+
                           " Database as "+ConnectionParams.s_userName);
   } catch(Exception ex){ //Trap SQL errors
     m_GUI.putStatus(
              "Error in Connecting to the Database "+'\n'+ex.toString());
   }
  }


  /**
  * Method to drop table BFILE_TABLE if it already exists. Then it creates the
  * table BFILE_TABLE (BFILE_COLUMN BFILE). This Table is used to store a row
  * with image file as a BFILE Column in that row.
  **/
  public void createTable() {
    try{
      // Create a Statement Object
      Statement l_stmt = m_connection.createStatement();

      // Use the Statement Object to execute query to DROP table BFILE_TABLE
      l_stmt.executeQuery(" DROP TABLE BFILE_TABLE ");

      l_stmt.close(); // Close the Statement Object

    } catch (SQLException ex) { //Trap SQL Errors
      m_GUI.putStatus(" Error while Dropping Table " + ex.toString());
    }
    try{
      // Create a Statement Object
      Statement l_stmt = m_connection.createStatement();

      // Use the Statement Object to execute query to CREATE table BFILE_TABLE
      l_stmt.executeQuery(" CREATE TABLE BFILE_TABLE(BFILE_COLUMN BFILE)");

      l_stmt.close(); // Close the Statement Object

      m_GUI.putStatus("Created table BFILE_TABLE.  Inserting Image Data..");
    } catch (SQLException ex) { //Trap SQL Errors
      m_GUI.putStatus(" Error while Creating Table " + ex.toString());
    }

  }

  /**
  * Method to insert a single record consisting of an image file ('architect.gif')
  * as BFILE column into the table BFILE_TABLE.
  * Please copy the file 'architect.gif' provided along with the sample to the
  * machine where database is running.
  **/
  public void insertData() {
    // Prompt the user to enter the directory Path where he/she has copied
    // the image file architect.gif
    String l_dirPath = m_GUI.getDirectoryPath();
    if (!l_dirPath.equals("Cancel")) // If user has entered Directory Path
      try{
        // Prepare a SQL Statement to Create Directory
        PreparedStatement l_pstmt = m_connection.prepareStatement(
            "CREATE OR REPLACE DIRECTORY BFILEDIR AS '" + l_dirPath + "' ");
        l_pstmt.execute(); // Execute the prepared SQL Statement

        // Insert the Image as a BFILE Column
        l_pstmt.executeQuery(" INSERT INTO BFILE_TABLE VALUES " +
             "( BFILENAME('BFILEDIR', 'architect.gif'))");

        l_pstmt.close(); // Close the prepared Statement
        m_GUI.putStatus("Created and populated BFILE_TABLE\n" +
         "Press SELECT to select image" );
        m_GUI.m_selectButton.setEnabled(true);
      } catch (SQLException ex)  { // Trap SQL Errors
        m_GUI.putStatus(" Error while inserting BFILE Data " + ex.toString());
      }
    else {
      m_GUI.putStatus("User Pressed Cancel Button. Exiting the application..");
      exitApplication();
    }
  }

  /**
  * Method to Select the BFILE Locator from the BFILE_TABLE and invokes the
  * displayFile() method to display the image. Note that BFILE Columns in
  * Oracle8i store the locator to the external files and not the actual File Data.
  **/
  public void selectBFILE(){
    try {
      // Create Statement Object
      Statement l_stmt = m_connection.createStatement();

      // Execute the Query and get BFILE Locator as a result set
      ResultSet l_rset = l_stmt.executeQuery("Select * from BFILE_TABLE ");

      //Note : Using Oracle Extension oracle.sql.BFILE to get BFILE Locator
      oracle.sql.BFILE  l_bfile = null;

      // Loop through the Result Set
      while(l_rset.next()) {
       l_bfile = ((OracleResultSet)l_rset).getBFILE(1); // Get the BFILE Locator
      }
      m_GUI.putStatus("Retrieved BFILE Locator..");

      l_rset.close(); // Close the ResultSet
      l_stmt.close(); // Close the Statement

      // Open the file with openFile Method of oracle.sql.BFILE class
      l_bfile.openFile();

      // Open the Input Binary Stream with  getBinaryStream method of
      // oracle.sql.BFILE class
      InputStream l_instream = l_bfile.getBinaryStream();
      m_GUI.appendStatus("Opened Binary Stream ...");
      m_GUI.displayFile(l_instream); // Invoke displayFile Method to display the image

    } catch (SQLException ex) { // Trap SQL errors
      m_GUI.putStatus(" Error While Selecting BFILE Locator " + ex.toString());
    }
  }

  /**
  * Close connection and exit application
  **/
  public void exitApplication(){
    try {
      if (m_connection != null)
        m_connection.close();
    } catch (SQLException ex){
      m_GUI.putStatus("Error while closing Connection \n"+ex.toString());
    }
    System.exit(0);
  }
}
