/**
 * @author  rsundara.in
 * @version 1.0
 *
 * Development Environment        :  JDeveloper 2.0
 * Name of the Application        :  StreamSample.java
 * Creation/Modification History  :
 *
 * rsundara.in       07-Jan-1999     Created
 *
 * Overview of Application        :
 *
 * This sample illustrates how data can be accessed through streams in JDBC.
 * Using JDBC stream data can be read in three formats:
 *
 *        1. Unicode
 *        2. Binary
 *        3. ASCII
 *
 * This sample reads a sample text file contents from the database, in the
 * chosen stream format.
 *
 * The GUI for this sample is handled in StreamFrame.java
 *
 */

import java.io.*;
import java.util.*;

import java.sql.*;  //Package for JDBC Classes.

public class StreamSample {

  Connection m_connection = null;  //Database Connection Object
  StreamFrame m_GUI;   // The GUI object for this sample

  /**
  * Constructor. Initializes the JTable and other GUI components.
  **/
  public StreamSample(){
    try{
       m_GUI=new StreamFrame(this);
       m_GUI.setVisible(true);
    }
    catch (Exception ex){ //Trap General Errors
      m_GUI.putStatus("Error...!!! "+ ex.toString());
     }
  }

  /**
  *  The main entry point to the application. Instantiates root frame
  *  and database connection is made here.
  **/
  public static void main(String args[]){
    StreamSample SS=new StreamSample(); //Instantiates the root Frame
    SS.dbConnection();                  //Establishes a DB connection
  }

  /**
  * Dispatches the GUI events to the appropriate method, which performs
  * the required JDBC operations. This method is invoked when event occurs
  * in the GUI (like  Button clicks etc.). This method is invoked from the
  * setupListeners section of StreamFrame.java
  **/
  public void dispatchEvent(String p_eventName) {
    // Dispatch Event
    if (p_eventName.equals("ASCII"))
      readAscii();
    else if(p_eventName.equals("BINARY"))
      readBinary();
    else if(p_eventName.equals("UNICODE"))
      readUnicode();
    else if (p_eventName.equals("EXIT"))
      exitApplication();

  }

  /**
  *  Creates a database connection object using JDBC. Please substitute the
  *  database connection parameters with appropriate values in
  *  ConnectionParams.java
  **/
  public void dbConnection(){

    try{

      m_GUI.putStatus("Trying to connect to the Database");

      // Load the Oracle JDBC Driver and register it.
      DriverManager.registerDriver(new oracle.jdbc.driver.OracleDriver());

      // Form the database connect string(TNSNAMES entry) as a name-value pair
      // using the connection parameters as specified in ConnectionParams.java
      String l_dbConnectString =
          "(DESCRIPTION=(ADDRESS=(HOST="+ConnectionParams.s_hostName+")"+
          "(PROTOCOL=tcp)(PORT="+ConnectionParams.s_portNumber+"))"+
          "(CONNECT_DATA=(SID="+ConnectionParams.s_databaseSID+")))";

      // *** The following statement creates a database connection object
      // using the DriverManager.getConnection method. The first parameter is
      // the database URL which is constructed based on the connection parameters
      // specified in ConnectionParams.java.
      // The URL syntax is as follows:
      //      "jdbc:oracle:<driver>:@<db connection string>"
      // <driver>, can be 'thin' or 'oci8'
      // <db connect string>, is a Net8 name-value, denoting the TNSNAMES entry
      m_connection = DriverManager.getConnection(
                  "jdbc:oracle:thin:@"+l_dbConnectString,
                  ConnectionParams.s_userName,
                  ConnectionParams.s_password);

      // sets the auto-commit property for the connection to be false. By default
      // the connections always auto-commit.
      m_connection.setAutoCommit(false);

      m_GUI.putStatus("Connected to "+ConnectionParams.s_databaseSID+
                           " Database as "+ConnectionParams.s_userName);
    } catch(Exception ex){ //Trap SQL errors
      m_GUI.putStatus(
              "Error in Connecting to the Database "+'\n'+ex.toString());
    }
  }


  /**
  *  This method areads text from the db and displays it in ascii
  *  format. This method used is getAsciiStream(), for reading the text from the db
  *  in ascii format
  **/
  public void readAscii(){
   try{
     // Display the file contents in TextArea and get the text to be converted
     String l_str=m_GUI.displayText(10);

     // The query selects the file contents from the database, using a
     // select <text> from dual SQL command
     String l_query=" SELECT "+"'"+l_str+"'"+" FROM dual ";

     //Create a Statement to execute the above SQL
     Statement l_statement =  m_connection.createStatement();

     // Execute the SQL and obtain the resultset
     ResultSet l_resultset=l_statement.executeQuery(l_query);

     // Retrieve stream from the resultset
     if(l_resultset.next()){
        //Open the ascii stream
        InputStream l_stream=l_resultset.getAsciiStream(1);

        int l_read; // Character read from stream
        int l_cnt=0;  // Counter to enable wrapping after 10 chars.

        // Loop through the stream, read each character and print it in GUI
        while((l_read=l_stream.read()) != -1){
          m_GUI.padAndAppend(l_read,"|"); // Print the obtained character in GUI

          // Wrap after 10 characters
          ++l_cnt;
          if (l_cnt==10) {
            l_cnt=0;
            m_GUI.m_textarea1.append("\n");
          }
        }
        l_stream.close();      //Close the InputStream
     }

     m_GUI.clearStatus();
     l_statement.close();   //Close the Statement object

   } catch(Exception ex){ // Trap IO and SQL errors
     m_GUI.putStatus("The exception is " + ex.toString());
   }
  }

  /**
  *  This method reads string from the db and displays in the binary
  *  format. This method used is getBinaryStream(), for reading in binary format
  **/
  public void readBinary(){
   try{
     // Display the file contents in TextArea and get the text to be converted
     String l_str=m_GUI.displayText(10);

     // The query selects the file contents from the database, using a
     // select <text> from dual SQL command
     String l_query=" SELECT "+"'"+l_str+"'"+" FROM dual ";

     //Create a Statement to execute the above SQL
     Statement l_statement =  m_connection.createStatement();

     // Execute the SQL and obtain the resultset
     ResultSet l_resultset=l_statement.executeQuery(l_query);

     // Retrieve stream from the resultset
     if(l_resultset.next()){
        //Open the binary stream
        InputStream l_stream=l_resultset.getBinaryStream(1);

        int l_read; // Character read from stream
        int l_cnt=0;  // Counter to enable wrapping after 10 chars.

        // Loop through the stream, read each character and print it in GUI
        while((l_read=l_stream.read()) != -1){
          m_GUI.m_textarea1.append(Integer.toHexString(l_read)+"|");

          // Wrap after 10 characters
          ++l_cnt;
          if (l_cnt==10) {
            l_cnt=0;
            m_GUI.m_textarea1.append("\n");
          }
        }
        l_stream.close();    //Closing the InputStream
     }
     m_GUI.clearStatus();
     l_statement.close();   //Closing the Statement object

   }catch(Exception ex){
     m_GUI.putStatus("The exception is " + ex.toString());
   }
 }

  /**
  *  This method reads string from the db and displays in the unicode  format.
  *  This method used is getUnicodeStream(), for reading in unicode format                                                  *
  **/
  public void readUnicode(){
   try{
     // Display the file contents in TextArea and get the text to be converted
     String l_str=m_GUI.displayText(6);

     // The query selects the file contents from the database, using a
     // select <text> from dual SQL command
     String l_query=" SELECT "+"'"+l_str+"'"+" FROM dual ";

     //Create a Statement to execute the above SQL
     Statement l_statement =  m_connection.createStatement();

     // Execute the SQL and obtain the resultset
     ResultSet l_resultset=l_statement.executeQuery(l_query);

     // Retrieve stream from the resultset
     if(l_resultset.next()){
        //Open the Unicode stream
        InputStream l_stream=l_resultset.getUnicodeStream(1);

        int l_read; // The current character read
        int l_cnt=0;  // Count to wrap after 10 chars.

        // Loop through the stream, read each character and print it in GUI
        while((l_read=l_stream.read()) != -1){
          // Print the first byte of the Unicode character
          m_GUI.padAndAppend(l_read, " "); // Print in GUI

          // Retrieve and print the second byte of Unicode
          m_GUI.padAndAppend(l_stream.read(),"|");

          // Wrap after 10 characters
          ++l_cnt;
          if (l_cnt==6) {
            l_cnt=0;
            m_GUI.m_textarea1.append("\n");
          }
        }
        l_stream.close();      //Close the InputStream
     }

     m_GUI.clearStatus();
     l_statement.close();   //Close the Statement object

   } catch (Exception ex){ // Trap SQL and IO errors
     m_GUI.putStatus("The exception is " + ex.toString());
   }
  }

  /**
  *  This method closes the connection object and exits the application
  **/
  public void exitApplication() {

    if (m_connection != null) {
         try {
           m_connection.close();             // close the connection when exit
         } catch (SQLException ex) {       // Trap SQL errors
            m_GUI.putStatus(" Error in closing the connection: ");
            m_GUI.appendStatus(" "+ex.toString());
         }
    }
    System.exit(0);
  }
}
